@@ -110,11 +110,11 @@ namespace xt
 
         size_type size() const noexcept;
 
-        constexpr size_type dimension() const noexcept;
+        XTENSOR_CONSTEXPR_RETURN size_type dimension() const noexcept;
 
-        constexpr const inner_shape_type& shape() const noexcept;
-        constexpr const inner_strides_type& strides() const noexcept;
-        constexpr const inner_backstrides_type& backstrides() const noexcept;
+        XTENSOR_CONSTEXPR_RETURN const inner_shape_type& shape() const noexcept;
+        XTENSOR_CONSTEXPR_RETURN const inner_strides_type& strides() const noexcept;
+        XTENSOR_CONSTEXPR_RETURN const inner_backstrides_type& backstrides() const noexcept;
 
         template <class T>
         void fill(const T& value);
@@ -423,6 +423,7 @@ namespace xt
         xstrided_container& operator=(xstrided_container&&) = default;
 
         explicit xstrided_container(inner_shape_type&&, inner_strides_type&&) noexcept;
+        explicit xstrided_container(inner_shape_type&&, inner_strides_type&&, inner_backstrides_type&&, layout_type&&) noexcept;
 
         inner_shape_type& shape_impl() noexcept;
         const inner_shape_type& shape_impl() const noexcept;
@@ -433,6 +434,11 @@ namespace xt
         inner_backstrides_type& backstrides_impl() noexcept;
         const inner_backstrides_type& backstrides_impl() const noexcept;
 
+        layout_type& mutable_layout() noexcept
+        {
+            return m_layout;
+        }
+
     private:
 
         inner_shape_type m_shape;
@@ -487,7 +493,7 @@ namespace xt
      * Returns the number of dimensions of the container.
      */
     template <class D>
-    inline constexpr auto xcontainer<D>::dimension() const noexcept -> size_type
+    XTENSOR_CONSTEXPR_RETURN auto xcontainer<D>::dimension() const noexcept -> size_type
     {
         return shape().size();
     }
@@ -496,7 +502,7 @@ namespace xt
      * Returns the shape of the container.
      */
     template <class D>
-    constexpr inline auto xcontainer<D>::shape() const noexcept -> const inner_shape_type&
+    XTENSOR_CONSTEXPR_RETURN auto xcontainer<D>::shape() const noexcept -> const inner_shape_type&
     {
         return derived_cast().shape_impl();
     }
@@ -505,7 +511,7 @@ namespace xt
      * Returns the strides of the container.
      */
     template <class D>
-    constexpr inline auto xcontainer<D>::strides() const noexcept -> const inner_strides_type&
+    XTENSOR_CONSTEXPR_RETURN auto xcontainer<D>::strides() const noexcept -> const inner_strides_type&
     {
         return derived_cast().strides_impl();
     }
@@ -514,7 +520,7 @@ namespace xt
      * Returns the backstrides of the container.
      */
     template <class D>
-    constexpr inline auto xcontainer<D>::backstrides() const noexcept -> const inner_backstrides_type&
+    XTENSOR_CONSTEXPR_RETURN auto xcontainer<D>::backstrides() const noexcept -> const inner_backstrides_type&
     {
         return derived_cast().backstrides_impl();
     }
@@ -548,7 +554,7 @@ namespace xt
     {
         XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
-        size_type index = xt::data_offset<size_type>(strides(), static_cast<size_type>(args)...);
+        size_type index = xt::data_offset<size_type>(strides(), static_cast<std::ptrdiff_t>(args)...);
         return storage()[index];
     }
 
@@ -564,7 +570,7 @@ namespace xt
     {
         XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
-        size_type index = xt::data_offset<size_type>(strides(), static_cast<size_type>(args)...);
+        size_type index = xt::data_offset<size_type>(strides(), static_cast<std::ptrdiff_t>(args)...);
         return storage()[index];
     }
 
@@ -625,7 +631,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::unchecked(Args... args) -> reference
     {
-        size_type index = xt::unchecked_data_offset<size_type>(strides(), static_cast<size_type>(args)...);
+        size_type index = xt::unchecked_data_offset<size_type, static_layout>(strides(), static_cast<std::ptrdiff_t>(args)...);
         return storage()[index];
     }
 
@@ -652,7 +658,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::unchecked(Args... args) const -> const_reference
     {
-        size_type index = xt::unchecked_data_offset<size_type>(strides(), static_cast<size_type>(args)...);
+        size_type index = xt::unchecked_data_offset<size_type, static_layout>(strides(), static_cast<std::ptrdiff_t>(args)...);
         return storage()[index];
     }
 
@@ -1261,6 +1267,12 @@ namespace xt
         adapt_strides(m_shape, m_strides, m_backstrides);
     }
 
+    template <class D>
+    inline xstrided_container<D>::xstrided_container(inner_shape_type&& shape, inner_strides_type&& strides, inner_backstrides_type&& backstrides, layout_type&& layout) noexcept
+        : base_type(), m_shape(std::move(shape)), m_strides(std::move(strides)), m_backstrides(std::move(backstrides)), m_layout(std::move(layout))
+    {
+    }
+
     template <class D>
     inline auto xstrided_container<D>::shape_impl() noexcept -> inner_shape_type&
     {
@@ -1333,16 +1345,17 @@ namespace xt
     template <class S>
     inline void xstrided_container<D>::resize(S&& shape, bool force)
     {
-        if (m_shape.size() != shape.size() || !std::equal(std::begin(shape), std::end(shape), std::begin(m_shape)) || force)
+        std::size_t dim = shape.size();
+        if (m_shape.size() != dim || !std::equal(std::begin(shape), std::end(shape), std::begin(m_shape)) || force)
         {
-            if (m_layout == layout_type::dynamic || m_layout == layout_type::any)
+            if (D::static_layout == layout_type::dynamic && m_layout == layout_type::dynamic)
             {
                 m_layout = XTENSOR_DEFAULT_LAYOUT;  // fall back to default layout
             }
             m_shape = xtl::forward_sequence<shape_type>(shape);
-            resize_container(m_strides, m_shape.size());
-            resize_container(m_backstrides, m_shape.size());
-            size_type data_size = compute_strides(m_shape, m_layout, m_strides, m_backstrides);
+            resize_container(m_strides, dim);
+            resize_container(m_backstrides, dim);
+            size_type data_size = compute_strides<D::static_layout>(m_shape, m_layout, m_strides, m_backstrides);
             detail::resize_data_container(this->storage(), data_size);
         }
     }
@@ -1399,11 +1412,11 @@ namespace xt
         {
             throw std::runtime_error("Cannot reshape with incorrect number of elements. Do you mean to resize?");
         }
-        if (layout == layout_type::dynamic || layout == layout_type::any)
+        if (D::static_layout == layout_type::dynamic && layout == layout_type::dynamic)
         {
             layout = XTENSOR_DEFAULT_LAYOUT;  // fall back to default layout
         }
-        if (layout != base_type::static_layout && base_type::static_layout != layout_type::dynamic)
+        if (D::static_layout != layout_type::dynamic && layout != D::static_layout)
         {
             throw std::runtime_error("Cannot reshape with different layout if static layout != dynamic.");
         }
@@ -1411,7 +1424,7 @@ namespace xt
         m_shape = xtl::forward_sequence<shape_type>(shape);
         resize_container(m_strides, m_shape.size());
         resize_container(m_backstrides, m_shape.size());
-        compute_strides(m_shape, m_layout, m_strides, m_backstrides);
+        compute_strides<D::static_layout>(m_shape, m_layout, m_strides, m_backstrides);
     }
 }
 
