@@ -35,18 +35,19 @@ namespace xt
     struct NAME                                                                 \
     {                                                                           \
         template <class U, class RT>                                            \
-        using frt = xt::detail::functor_return_type<U, RT>;                     \
-        using return_type = frt<T, R>;                                          \
+        using fvt_t = xt::detail::functor_value_type_t<U, RT>;                  \
+        template <class U, class RT>                                            \
+        using fst_t = xt::detail::functor_batch_simd_type_t<U, RT>;             \
         using argument_type = T;                                                \
-        using result_type = typename return_type::type;                         \
+        using result_type = fvt_t<T, R>;                                        \
         using simd_value_type = xsimd::simd_type<T>;                            \
-        using simd_result_type = typename return_type::simd_type;               \
+        using simd_result_type = fst_t<simd_value_type, R>;                     \
         constexpr result_type operator()(const T& arg) const                    \
         {                                                                       \
             return OP arg;                                                      \
         }                                                                       \
         template <class B>                                                      \
-        constexpr typename frt<get_value_type_t<B>, R>::simd_type               \
+        constexpr fst_t<B, R>                                                   \
         simd_apply(const B& arg) const                                          \
         {                                                                       \
             return OP arg;                                                      \
@@ -72,20 +73,21 @@ namespace xt
     struct NAME                                                                  \
     {                                                                            \
         template <class U, class RT>                                             \
-        using frt = xt::detail::functor_return_type<U, RT>;                      \
-        using return_type = frt<T, R>;                                           \
+        using fvt_t = xt::detail::functor_value_type_t<U, RT>;                   \
+        template <class U, class RT>                                             \
+        using fst_t = xt::detail::functor_batch_simd_type_t<U, RT>;              \
         using first_argument_type = T;                                           \
         using second_argument_type = T;                                          \
-        using result_type = typename return_type::type;                          \
+        using result_type = fvt_t<T, R>;                                         \
         using simd_value_type = xsimd::simd_type<T>;                             \
-        using simd_result_type = typename return_type::simd_type;                \
+        using simd_result_type = fst_t<simd_value_type, R>;                      \
         template <class T1, class T2>                                            \
         constexpr result_type operator()(const T1& arg1, const T2& arg2) const   \
         {                                                                        \
             return (arg1 OP arg2);                                               \
         }                                                                        \
         template <class B>                                                       \
-        constexpr typename frt<get_value_type_t<B>, R>::simd_type                \
+        constexpr fst_t<B, R>                                                    \
         simd_apply(const B& arg1, const B& arg2) const                           \
         {                                                                        \
             return (arg1 OP arg2);                                               \
@@ -161,11 +163,10 @@ namespace xt
             template <class T>
             struct functor
             {
-                using return_type = xt::detail::functor_return_type<T, R>;
                 using argument_type = T;
-                using result_type = typename return_type::type;
+                using result_type = R;
                 using simd_value_type = xsimd::simd_type<T>;
-                using simd_result_type = typename return_type::simd_type;
+                using simd_result_type = xt::detail::functor_simd_type_t<T, R>;
                 constexpr result_type operator()(const T& arg) const
                 {
                     return static_cast<R>(arg);
