@@ -22,10 +22,38 @@
 #include "xiterable.hpp"
 #include "xlayout.hpp"
 #include "xsemantic.hpp"
+#include "xstorage.hpp"
 #include "xstrided_view_base.hpp"
 
 namespace xt
 {
+    namespace detail
+    {
+        template <class S>
+        struct xtype_for_shape
+        {
+            template <class T, layout_type L>
+            using type = xarray<T, L>;
+        };
+
+        template <template <class, std::size_t> class S, class X, std::size_t N>
+        struct xtype_for_shape<S<X, N>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor<T, N, L>;
+        };
+
+        template <template <std::size_t...> class S, std::size_t... X>
+        struct xtype_for_shape<S<X...>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor_fixed<T, xshape<X...>, L>;
+        };
+
+        template <class T, class S, layout_type L>
+        using temporary_type_t = typename xtype_for_shape<S>::template type<T, L>;
+    }
+
     template <class CT, class S, layout_type L, class FST>
     class xstrided_view;
 
@@ -33,15 +61,15 @@ namespace xt
     struct xcontainer_inner_types<xstrided_view<CT, S, L, FST>>
     {
         using xexpression_type = std::decay_t<CT>;
-        using temporary_type = xarray<std::decay_t<typename xexpression_type::value_type>>;
+        using temporary_type = detail::temporary_type_t<typename xexpression_type::value_type, S, L>;
     };
 
     template <class CT, class S, layout_type L, class FST>
     struct xiterable_inner_types<xstrided_view<CT, S, L, FST>>
     {
         using inner_shape_type = S;
-        using inner_strides_type = inner_shape_type;
-        using inner_backstrides_type = inner_shape_type;
+        using inner_strides_type = get_strides_t<inner_shape_type>;
+        using inner_backstrides_type_type = inner_strides_type;
 
         using const_stepper = std::conditional_t<
             is_indexed_stepper<typename std::decay_t<CT>::stepper>::value,
@@ -66,8 +94,9 @@ namespace xt
      * and strides.
      *
      * @tparam CT the closure type of the \ref xexpression type underlying this view
+     * @tparam L the layout of the strided view
      * @tparam S the strides type of the strided view
-     * @tparam FST the flat storage type used for the strided view.
+     * @tparam FST the flat storage type used for the strided view
      *
      * @sa strided_view, transpose
      */
@@ -99,8 +128,8 @@ namespace xt
         using iterable_base = xiterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = typename base_type::shape_type;
-        using strides_type = typename base_type::strides_type;;
-        using backstrides_type = typename base_type::backstrides_type;;
+        using strides_type = typename base_type::strides_type;
+        using backstrides_type = typename base_type::backstrides_type;
 
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
@@ -114,10 +143,13 @@ namespace xt
         using simd_value_type = xsimd::simd_type<value_type>;
 
         template <class CTA>
-        xstrided_view(CTA&& e, S&& shape, S&& strides, std::size_t offset, layout_type layout) noexcept;
+        xstrided_view(CTA&& e, S&& shape, strides_type&& strides, std::size_t offset, layout_type layout) noexcept;
 
         template <class CTA, class FLS>
-        xstrided_view(CTA&& e, S&& shape, S&& strides, std::size_t offset, layout_type layout, FLS&& flatten_strides, layout_type flatten_layout) noexcept;
+        xstrided_view(CTA&& e, S&& shape, strides_type&& strides, std::size_t offset,
+                      layout_type layout, FLS&& flatten_strides, layout_type flatten_layout) noexcept;
+
+        xstrided_view(const xstrided_view& rhs) = default;
 
         template <class E>
         self_type& operator=(const xexpression<E>& e);
@@ -172,18 +204,13 @@ namespace xt
                                                       typename storage_type::iterator>;
         using const_container_iterator = typename storage_type::const_iterator;
 
-    protected:
+    private:
 
         container_iterator data_xbegin() noexcept;
         const_container_iterator data_xbegin() const noexcept;
         container_iterator data_xend(layout_type l) noexcept;
         const_container_iterator data_xend(layout_type l) const noexcept;
 
-    private:
-
-        template <class C>
-        friend class xstepper;
-
         template <class It>
         It data_xbegin_impl(It begin) const noexcept;
 
@@ -192,6 +219,8 @@ namespace xt
 
         void assign_temporary_impl(temporary_type&& tmp);
 
+        template <class C>
+        friend class xstepper;
         friend class xview_semantic<xstrided_view<CT, S, L, FST>>;
     };
 
@@ -220,16 +249,16 @@ namespace xt
     >;
 
     /**
-     * @typedef xstrided_slice_vector
-     * @brief vector of slices used to build a `xstrided_view`
-     */
+    * @typedef xstrided_slice_vector
+    * @brief vector of slices used to build a `xstrided_view`
+    */
     using xstrided_slice_vector = std::vector<xstrided_slice<std::ptrdiff_t>>;
 
     // TODO: remove this type used for backward compatibility only
     using slice_vector = xstrided_slice_vector;
 
-    template <layout_type L = layout_type::dynamic, class E, class I>
-    auto strided_view(E&& e, I&& shape, I&& strides, std::size_t offset = 0, layout_type layout = layout_type::dynamic) noexcept;
+    template <layout_type L = layout_type::dynamic, class E, class S, class X>
+    auto strided_view(E&& e, S&& shape, X&& stride, std::size_t offset = 0, layout_type layout = L) noexcept;
 
     template <class E>
     auto strided_view(E&& e, const xstrided_slice_vector& slices);
@@ -274,14 +303,14 @@ namespace xt
      */
     template <class CT, class S, layout_type L, class FST>
     template <class CTA>
-    inline xstrided_view<CT, S, L, FST>::xstrided_view(CTA&& e, S&& shape, S&& strides, std::size_t offset, layout_type layout) noexcept
+    inline xstrided_view<CT, S, L, FST>::xstrided_view(CTA&& e, S&& shape, strides_type&& strides, std::size_t offset, layout_type layout) noexcept
         : base_type(std::forward<CTA>(e), std::move(shape), std::move(strides), offset, layout)
     {
     }
 
     template <class CT, class S, layout_type L, class FST>
     template <class CTA, class FLS>
-    inline xstrided_view<CT, S, L, FST>::xstrided_view(CTA&& e, S&& shape, S&& strides, std::size_t offset,
+    inline xstrided_view<CT, S, L, FST>::xstrided_view(CTA&& e, S&& shape, strides_type&& strides, std::size_t offset,
                                                        layout_type layout, FLS&& flatten_strides, layout_type flatten_layout) noexcept
         : base_type(std::forward<CTA>(e), std::move(shape), std::move(strides), offset, layout, std::forward<FLS>(flatten_strides), flatten_layout)
     {
@@ -456,43 +485,16 @@ namespace xt
      *
      * @tparam L the static layout type of the view (default: dynamic)
      * @tparam E type of xexpression
-     * @tparam I shape and strides type
+     * @tparam S strides type
+     * @tparam X strides type
      *
      * @return the view
      */
-    template <layout_type L, class E, class I>
-    inline auto strided_view(E&& e, I&& shape, I&& strides, std::size_t offset, layout_type layout) noexcept
+    template <layout_type L, class E, class S, class X>
+    inline auto strided_view(E&& e, S&& shape, X&& strides, std::size_t offset, layout_type layout) noexcept
     {
-        using view_type = xstrided_view<xclosure_t<E>, I, L>;
-        return view_type(std::forward<E>(e), std::forward<I>(shape), std::forward<I>(strides), offset, layout);
-    }
-
-    namespace detail
-    {
-        template <class S>
-        struct slice_getter_impl
-        {
-            const S& m_shape;
-            mutable std::size_t idx;
-
-            slice_getter_impl(const S& shape)
-                : m_shape(shape), idx(0)
-            {
-            }
-
-            template <class T>
-            std::array<std::ptrdiff_t, 3> operator()(const T& /*t*/) const
-            {
-                return {0, 0, 0};
-            }
-
-            template <class A, class B, class C>
-            std::array<std::ptrdiff_t, 3> operator()(const xrange_adaptor<A, B, C>& range) const
-            {
-                auto sl = range.get(static_cast<std::size_t>(m_shape[idx]));
-                return {sl(0), sl.size(), sl.step_size()};
-            }
-        };
+        using view_type = xstrided_view<xclosure_t<E>, S, L>;
+        return view_type(std::forward<E>(e), std::forward<S>(shape), std::forward<X>(strides), offset, layout);
     }
 
     template <class T>
@@ -507,126 +509,22 @@ namespace xt
 
     namespace detail
     {
-        template <class S, class ST>
-        inline auto get_strided_view_args(const S& shape, ST&& strides, std::size_t base_offset, layout_type layout, const xstrided_slice_vector& slices)
+        struct no_adj_strides_policy
         {
-            // Compute dimension
-            std::size_t dimension = shape.size(), n_newaxis = 0, n_add_all = 0;
-            std::ptrdiff_t dimension_check = static_cast<std::ptrdiff_t>(shape.size());
+        protected:
 
-            bool has_ellipsis = false;
-            for (const auto& el : slices)
+            inline void resize(std::size_t) {}
+            inline void set_fake_slice(std::size_t) {}
+
+            template <class ST, class S>
+            bool fill_args(const xstrided_slice_vector& /*slices*/, std::size_t /*sl_idx*/,
+                           std::size_t /*i*/, std::size_t /*old_shape*/,
+                           const ST& /*old_stride*/,
+                           S& /*shape*/, get_strides_t<S>& /*strides*/)
             {
-                if (xtl::get_if<xt::xnewaxis_tag>(&el) != nullptr)
-                {
-                    ++dimension;
-                    ++n_newaxis;
-                }
-                else if (xtl::get_if<std::ptrdiff_t>(&el) != nullptr)
-                {
-                    --dimension;
-                    --dimension_check;
-                }
-                else if (xtl::get_if<xt::xellipsis_tag>(&el) != nullptr)
-                {
-                    if (has_ellipsis == true)
-                    {
-                        throw std::runtime_error("Ellipsis can only appear once.");
-                    }
-                    has_ellipsis = true;
-                }
-                else
-                {
-                    --dimension_check;
-                }
+                return false;
             }
-
-            if (dimension_check < 0)
-            {
-                throw std::runtime_error("Too many slices for view.");
-            }
-
-            if (has_ellipsis)
-            {
-                // replace ellipsis with N * xt::all
-                // remove -1 because of the ellipsis slize itself
-                n_add_all = shape.size() - (slices.size() - 1 - n_newaxis);
-            }
-
-            // Compute strided view
-            std::size_t offset = base_offset;
-            using shape_type = dynamic_shape<std::ptrdiff_t>;
-
-            shape_type new_shape(dimension);
-            shape_type new_strides(dimension);
-
-            auto old_shape = shape;
-            auto&& old_strides = strides;
-
-            using old_strides_vt = std::decay_t<decltype(old_strides[0])>;
-
-    #define XTENSOR_MS(v) static_cast<std::ptrdiff_t>(v)
-    #define XTENSOR_MU(v) static_cast<std::size_t>(v)
-
-            std::ptrdiff_t i = 0, axis_skip = 0;
-            std::size_t idx = 0;
-
-            auto slice_getter = detail::slice_getter_impl<S>(shape);
-
-            for (; i < XTENSOR_MS(slices.size()); ++i)
-            {
-                auto ptr = xtl::get_if<std::ptrdiff_t>(&slices[XTENSOR_MU(i)]);
-                if (ptr != nullptr)
-                {
-                    auto slice0 = static_cast<old_strides_vt>(*ptr);
-                    offset += static_cast<std::size_t>(slice0 * old_strides[XTENSOR_MU(i - axis_skip)]);
-                }
-                else if (xtl::get_if<xt::xnewaxis_tag>(&slices[XTENSOR_MU(i)]) != nullptr)
-                {
-                    new_shape[idx] = 1;
-                    ++axis_skip, ++idx;
-                }
-                else if (xtl::get_if<xt::xellipsis_tag>(&slices[XTENSOR_MU(i)]) != nullptr)
-                {
-                    for (std::size_t j = 0; j < n_add_all; ++j)
-                    {
-                        new_shape[idx] = XTENSOR_MS(old_shape[XTENSOR_MU(i - axis_skip)]);
-                        new_strides[idx] = XTENSOR_MS(old_strides[XTENSOR_MU(i - axis_skip)]);
-                        --axis_skip, ++idx;
-                    }
-                    ++axis_skip;  // because i++
-                }
-                else if (xtl::get_if<xt::xall_tag>(&slices[XTENSOR_MU(i)]) != nullptr)
-                {
-                    new_shape[idx] = XTENSOR_MS(old_shape[XTENSOR_MU(i - axis_skip)]);
-                    new_strides[idx] = XTENSOR_MS(old_strides[XTENSOR_MU(i - axis_skip)]);
-                    ++idx;
-                }
-                else
-                {
-                    slice_getter.idx = XTENSOR_MU(i - axis_skip);
-                    auto info = xtl::visit(slice_getter, slices[XTENSOR_MU(i)]);
-                    offset += XTENSOR_MU(static_cast<old_strides_vt>(info[0]) * old_strides[XTENSOR_MU(i - axis_skip)]);
-                    new_shape[idx] = std::ptrdiff_t(info[1]);
-                    new_strides[idx] = std::ptrdiff_t(info[2]) * XTENSOR_MS(old_strides[XTENSOR_MU(i - axis_skip)]);
-                    ++idx;
-                }
-            }
-
-            for (; XTENSOR_MU(i - axis_skip) < old_shape.size(); ++i)
-            {
-                new_shape[idx] = XTENSOR_MS(old_shape[XTENSOR_MU(i - axis_skip)]);
-                new_strides[idx] = XTENSOR_MS(old_strides[XTENSOR_MU(i - axis_skip)]);
-                ++idx;
-            }
-
-            layout_type new_layout = do_strides_match(new_shape, new_strides, layout) ? layout : layout_type::dynamic;
-
-            return std::make_tuple(std::move(new_shape), std::move(new_strides), offset, new_layout);
-
-    #undef XTENSOR_MU
-    #undef XTENSOR_MS
-        }
+        };
     }
 
     /**
@@ -657,25 +555,28 @@ namespace xt
     template <class E>
     inline auto strided_view(E&& e, const xstrided_slice_vector& slices)
     {
-        auto args = detail::get_strided_view_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
-        using view_type = xstrided_view<xclosure_t<E>, std::decay_t<decltype(std::get<0>(args))>>;
-        return view_type(std::forward<E>(e), std::move(std::get<0>(args)), std::move(std::get<1>(args)), std::get<2>(args), std::get<3>(args));
+        detail::strided_view_args<detail::no_adj_strides_policy> args;
+        args.fill_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
+        using view_type = xstrided_view<xclosure_t<E>, decltype(args.new_shape)>;
+        return view_type(std::forward<E>(e), std::move(args.new_shape), std::move(args.new_strides), args.new_offset, args.new_layout);
     }
 
     template <class CT, class S, layout_type L, class FST>
     auto strided_view(const xstrided_view<CT, S, L, FST>& e, const xstrided_slice_vector& slices)
     {
-        auto args = detail::get_strided_view_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
-        using view_type = xstrided_view<xclosure_t<decltype(e.expression())>, std::decay_t<decltype(std::get<0>(args))>>;
-        return view_type(e.expression(), std::move(std::get<0>(args)), std::move(std::get<1>(args)), std::get<2>(args), std::get<3>(args));
+        detail::strided_view_args<detail::no_adj_strides_policy> args;
+        args.fill_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
+        using view_type = xstrided_view<xclosure_t<decltype(e.expression())>, decltype(args.new_shape)>;
+        return view_type(e.expression(), std::move(args.new_shape), std::move(args.new_strides), args.new_offset, args.new_layout);
     }
 
     template <class CT, class S, layout_type L, class FST>
     auto strided_view(xstrided_view<CT, S, L, FST>& e, const xstrided_slice_vector& slices)
     {
-        auto args = detail::get_strided_view_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
-        using view_type = xstrided_view<xclosure_t<decltype(e.expression())>, std::decay_t<decltype(std::get<0>(args))>>;
-        return view_type(e.expression(), std::move(std::get<0>(args)), std::move(std::get<1>(args)), std::get<2>(args), std::get<3>(args));
+        detail::strided_view_args<detail::no_adj_strides_policy> args;
+        args.fill_args(e.shape(), detail::get_strides(e), detail::get_offset(e), e.layout(), slices);
+        using view_type = xstrided_view<xclosure_t<decltype(e.expression())>, decltype(args.new_shape)>;
+        return view_type(e.expression(), std::move(args.new_shape), std::move(args.new_strides), args.new_offset, args.new_layout);
     }
 
     /****************************
@@ -716,14 +617,14 @@ namespace xt
             }
 
             // permute stride and shape
-            using strides_type = typename std::decay_t<E>::strides_type;
-            strides_type temp_strides;
-            resize_container(temp_strides, e.strides().size());
-
-            using shape_type = typename std::decay_t<E>::shape_type;
+            using shape_type = xindex_type_t<typename std::decay_t<E>::shape_type>;
             shape_type temp_shape;
             resize_container(temp_shape, e.shape().size());
 
+            using strides_type = get_strides_t<shape_type>;
+            strides_type temp_strides;
+            resize_container(temp_strides, e.strides().size());
+
             using size_type = typename std::decay_t<E>::size_type;
             for (std::size_t i = 0; i < e.shape().size(); ++i)
             {
@@ -768,14 +669,14 @@ namespace xt
             return transpose_impl(std::forward<E>(e), std::forward<S>(permutation), check_policy::none());
         }
 
-        template <class E, class S, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline void compute_transposed_strides(E&& e, const S&, S& strides)
+        template <class E, class S, class X, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
+        inline void compute_transposed_strides(E&& e, const S&, X& strides)
         {
             std::copy(e.strides().crbegin(), e.strides().crend(), strides.begin());
         }
 
-        template <class E, class S, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline void compute_transposed_strides(E&&, const S& shape, S& strides)
+        template <class E, class S, class X, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
+        inline void compute_transposed_strides(E&&, const S& shape, X& strides)
         {
             layout_type l = transpose_layout(std::decay_t<E>::static_layout);
             compute_strides(shape, l, strides);
@@ -789,12 +690,12 @@ namespace xt
     template <class E>
     inline auto transpose(E&& e) noexcept
     {
-        using shape_type = typename std::decay_t<E>::shape_type;
+        using shape_type = xindex_type_t<typename std::decay_t<E>::shape_type>;
         shape_type shape;
         resize_container(shape, e.shape().size());
         std::copy(e.shape().crbegin(), e.shape().crend(), shape.begin());
 
-        shape_type strides;
+        get_strides_t<shape_type> strides;
         resize_container(strides, e.shape().size());
         detail::compute_transposed_strides(e, shape, strides);
 
@@ -846,7 +747,8 @@ namespace xt
             using shape_type = static_shape<std::size_t, 1>;
             using view_type = xstrided_view<xclosure_t<E>, shape_type>;
 
-            shape_type new_shape, new_strides;
+            shape_type new_shape;
+            get_strides_t<shape_type> new_strides;
             new_shape[0] = e.size();
             new_strides[0] = std::size_t(1);
             std::size_t offset = detail::get_offset(e);
@@ -862,9 +764,10 @@ namespace xt
         inline auto build_ravel_view(E&& e, S&& flatten_strides, layout_type l)
         {
             using shape_type = static_shape<std::size_t, 1>;
-            using view_type = xstrided_view<xclosure_t<E>, shape_type, layout_type::dynamic, detail::flat_expression_adaptor<xclosure_t<E>>>;
+            using view_type = xstrided_view<xclosure_t<E>, shape_type, layout_type::dynamic, detail::flat_expression_adaptor<std::remove_reference_t<E>>>;
 
-            shape_type new_shape, new_strides;
+            shape_type new_shape;
+            get_strides_t<shape_type> new_strides;
             new_shape[0] = e.size();
             new_strides[0] = std::size_t(1);
             std::size_t offset = detail::get_offset(e);
@@ -895,8 +798,8 @@ namespace xt
             inline static auto run(E&& e)
             {
                 // Case where the static layout is either row_major or column major.
-                using shape_type = typename std::decay_t<E>::shape_type;
-                shape_type strides;
+                using shape_type = xindex_type_t<typename std::decay_t<E>::shape_type>;
+                get_strides_t<shape_type> strides;
                 resize_container(strides, e.shape().size());
                 layout_type l = detail::transpose_layout(e.layout());
                 compute_strides(e.shape(), l, strides);
@@ -971,14 +874,15 @@ namespace xt
     template <class E>
     inline auto squeeze(E&& e)
     {
-        xt::dynamic_shape<std::size_t> new_shape, new_strides;
+        dynamic_shape<std::size_t> new_shape;
+        dynamic_shape<std::ptrdiff_t> new_strides;
         std::copy_if(e.shape().cbegin(), e.shape().cend(), std::back_inserter(new_shape),
                      [](std::size_t i) { return i != 1; });
-        auto&& old_strides = detail::get_strides(e);
+        decltype(auto) old_strides = detail::get_strides(e);
         std::copy_if(old_strides.cbegin(), old_strides.cend(), std::back_inserter(new_strides),
-                     [](std::size_t i) { return i != 0; });
+                     [](std::ptrdiff_t i) { return i != 0; });
 
-        using view_type = xstrided_view<xclosure_t<E>, xt::dynamic_shape<std::size_t>>;
+        using view_type = xstrided_view<xclosure_t<E>, dynamic_shape<std::size_t>>;
         return view_type(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
     }
 
@@ -988,7 +892,8 @@ namespace xt
         inline auto squeeze_impl(E&& e, S&& axis, check_policy::none)
         {
             std::size_t new_dim = e.dimension() - axis.size();
-            xt::dynamic_shape<std::size_t> new_shape(new_dim), new_strides(new_dim);
+            dynamic_shape<std::size_t> new_shape(new_dim);
+            dynamic_shape<std::ptrdiff_t> new_strides(new_dim);
 
             decltype(auto) old_strides = detail::get_strides(e);
 
@@ -1001,7 +906,7 @@ namespace xt
                 }
             }
 
-            using view_type = xstrided_view<xclosure_t<E>, xt::dynamic_shape<std::size_t>>;
+            using view_type = xstrided_view<xclosure_t<E>, dynamic_shape<std::size_t>>;
             return view_type(std::forward<E>(e), std::move(new_shape), std::move(new_strides), 0, e.layout());
         }
 
@@ -1075,8 +980,8 @@ namespace xt
     template <class E>
     auto expand_dims(E&& e, std::size_t axis)
     {
-        xstrided_slice_vector sv(e.dimension() + 1, xt::all());
-        sv[axis] = xt::newaxis();
+        xstrided_slice_vector sv(e.dimension() + 1, all());
+        sv[axis] = newaxis();
         return strided_view(std::forward<E>(e), std::move(sv));
     }
 
@@ -1095,19 +1000,19 @@ namespace xt
     template <std::size_t N, class E>
     auto atleast_Nd(E&& e)
     {
-        xstrided_slice_vector sv((std::max)(e.dimension(), N), xt::all());
+        xstrided_slice_vector sv((std::max)(e.dimension(), N), all());
         if (e.dimension() < N)
         {
             std::size_t i = 0;
             std::size_t end = static_cast<std::size_t>(std::round(double(N - e.dimension()) / double(N)));
             for (; i < end; ++i)
             {
-                sv[i] = xt::newaxis();
+                sv[i] = newaxis();
             }
             i += e.dimension();
             for (; i < N; ++i)
             {
-                sv[i] = xt::newaxis();
+                sv[i] = newaxis();
             }
         }
         return strided_view(std::forward<E>(e), std::move(sv));
@@ -1164,7 +1069,7 @@ namespace xt
         }
 
         std::size_t ax_sz = e.shape()[axis];
-        xstrided_slice_vector sv(e.dimension(), xt::all());
+        xstrided_slice_vector sv(e.dimension(), all());
         std::size_t step = ax_sz / n;
         std::size_t rest = ax_sz % n;
 
@@ -1182,30 +1087,6 @@ namespace xt
         return result;
     }
 
-    template <class T>
-    struct make_signed_shape;
-
-    template <class E, std::size_t N>
-    struct make_signed_shape<std::array<E, N>>
-    {
-        using type = std::array<typename std::make_signed<E>::type, N>;
-    };
-
-    template <class E>
-    struct make_signed_shape<std::vector<E>>
-    {
-        using type = std::vector<typename std::make_signed<E>::type>;
-    };
-
-    template <class E>
-    struct make_signed_shape<xt::svector<E>>
-    {
-        using type = xt::svector<typename std::make_signed<E>::type>;
-    };
-
-    template <class T>
-    using make_signed_shape_t = typename make_signed_shape<T>::type;
-
     /**
      * @brief Reverse the order of elements in an xexpression along the given axis.
      * Note: A NumPy/Matlab style `flipud(arr)` is equivalent to `xt::flip(arr, 0)`,
@@ -1219,20 +1100,19 @@ namespace xt
     template <class E>
     inline auto flip(E&& e, std::size_t axis)
     {
-        using shape_type = typename std::decay_t<E>::shape_type;
-        using signed_shape_type = make_signed_shape_t<shape_type>;
+        using shape_type = xindex_type_t<typename std::decay_t<E>::shape_type>;
 
-        signed_shape_type shape;
+        shape_type shape;
         resize_container(shape, e.shape().size());
         std::copy(e.shape().cbegin(), e.shape().cend(), shape.begin());
 
-        signed_shape_type strides;
-        auto&& old_strides = detail::get_strides(e);
+        get_strides_t<shape_type> strides;
+        decltype(auto) old_strides = detail::get_strides(e);
         resize_container(strides, old_strides.size());
         std::copy(old_strides.cbegin(), old_strides.cend(), strides.begin());
 
         strides[axis] *= -1;
-        std::size_t offset = old_strides[axis] * (e.shape()[axis] - 1);
+        std::size_t offset = static_cast<std::size_t>(old_strides[axis] * (static_cast<std::ptrdiff_t>(e.shape()[axis]) - 1));
 
         return strided_view(std::forward<E>(e), std::move(shape), std::move(strides), offset);
     }
@@ -1240,12 +1120,11 @@ namespace xt
     template <class E, class S>
     inline auto reshape_view(E&& e, S&& shape)
     {
-        using shape_type = S;
+        using shape_type = std::decay_t<S>;
+        get_strides_t<shape_type> strides;
 
-        shape_type strides;
         xt::resize_container(strides, shape.size());
         compute_strides(shape, default_assignable_layout(std::decay_t<E>::static_layout), strides);
-
         return strided_view<std::decay_t<E>::static_layout>(std::forward<E>(e), std::forward<S>(shape), std::move(strides), 0);
     }
 
@@ -1264,9 +1143,9 @@ namespace xt
     template <class E, class S>
     inline auto reshape_view(E&& e, S&& shape, layout_type layout)
     {
-        using shape_type = S;
+        using shape_type = std::decay_t<S>;
+        get_strides_t<shape_type> strides;
 
-        shape_type strides;
         xt::resize_container(strides, shape.size());
         compute_strides(shape, layout, strides);
 
