@@ -9,6 +9,8 @@
 #ifndef XTENSOR_STRIDED_VIEW_BASE_HPP
 #define XTENSOR_STRIDED_VIEW_BASE_HPP
 
+#include <type_traits>
+
 #include <xtl/xsequence.hpp>
 
 #include "xtensor_forward.hpp"
@@ -42,24 +44,32 @@ namespace xt
         using storage_type = std::remove_reference_t<inner_storage_type>;
 
         using shape_type = S;
-        using strides_type = shape_type;
-        using backstrides_type = shape_type;
+        using strides_type = get_strides_t<shape_type>;
+        using backstrides_type = strides_type;
+
+        using inner_shape_type = shape_type;
+        using inner_strides_type = strides_type;
+        using inner_backstrides_type = backstrides_type;
 
         static constexpr layout_type static_layout = L;
-        static constexpr bool contiguous_layout = static_layout != layout_type::dynamic;
+        static constexpr bool contiguous_layout = static_layout != layout_type::dynamic && xexpression_type::contiguous_layout;
 
         template <class CTA>
-        xstrided_view_base(CTA&& e, S&& shape, S&& strides, std::size_t offset, layout_type layout) noexcept;
+        xstrided_view_base(CTA&& e, S&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept;
 
         template <class CTA, class FLS>
-        xstrided_view_base(CTA&& e, S&& shape, S&& strides, std::size_t offset,
+        xstrided_view_base(CTA&& e, S&& shape, strides_type&& strides, size_type offset,
                            layout_type layout, FLS&& flatten_strides, layout_type flatten_layout) noexcept;
 
+        xstrided_view_base(xstrided_view_base&& rhs);
+
+        xstrided_view_base(const xstrided_view_base& rhs);
+
         size_type size() const noexcept;
         size_type dimension() const noexcept;
-        const shape_type& shape() const noexcept;
-        const strides_type& strides() const noexcept;
-        const backstrides_type& backstrides() const noexcept;
+        const inner_shape_type& shape() const noexcept;
+        const inner_strides_type& strides() const noexcept;
+        const inner_backstrides_type& backstrides() const noexcept;
         layout_type layout() const noexcept;
 
         reference operator()();
@@ -121,14 +131,27 @@ namespace xt
         template <class O>
         bool is_trivial_broadcast(const O& strides) const noexcept;
 
+    protected:
+
+        using offset_type = typename strides_type::value_type;
+
+        template <class... Args>
+        offset_type compute_index(Args... args) const;
+
+        template <class... Args>
+        offset_type compute_unchecked_index(Args... args) const;
+
+        template <class It>
+        offset_type compute_element_index(It first, It last) const;
+
     private:
 
         CT m_e;
         inner_storage_type m_storage;
-        shape_type m_shape;
-        strides_type m_strides;
-        backstrides_type m_backstrides;
-        std::size_t m_offset;
+        inner_shape_type m_shape;
+        inner_strides_type m_strides;
+        inner_backstrides_type m_backstrides;
+        size_type m_offset;
         layout_type m_layout;
     };
 
@@ -154,14 +177,19 @@ namespace xt
             using iterator = typename xexpression_type::iterator;
             using const_iterator = typename xexpression_type::const_iterator;
 
-            flat_expression_adaptor(CT& e);
+            explicit flat_expression_adaptor(CT* e);
 
             template <class FST>
-            flat_expression_adaptor(CT& e, FST&& strides, layout_type layout);
+            flat_expression_adaptor(CT* e, FST&& strides, layout_type layout);
+
+            void update_pointer(CT* ptr) const
+            {
+                m_e = ptr;
+            }
 
             size_type size() const;
-            reference operator[](std::size_t idx);
-            const_reference operator[](std::size_t idx) const;
+            reference operator[](size_type idx);
+            const_reference operator[](size_type idx) const;
 
             iterator begin();
             iterator end();
@@ -172,7 +200,7 @@ namespace xt
 
         private:
 
-            CT& m_e;
+            mutable CT* m_e;
             shape_type m_strides;
             mutable index_type m_index;
             size_type m_size;
@@ -196,7 +224,7 @@ namespace xt
         template <class CT>
         struct flat_storage_type<CT, typename std::enable_if_t<!has_data_interface<std::decay_t<CT>>::value>>
         {
-            using type = flat_expression_adaptor<CT>;
+            using type = flat_expression_adaptor<std::remove_reference_t<CT>>;
         };
 
         template <class CT>
@@ -210,7 +238,7 @@ namespace xt
         }
 
         template <class E, std::enable_if_t<has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline std::size_t get_offset(E&& e)
+        inline auto get_offset(E&& e)
         {
             return e.data_offset();
         }
@@ -223,27 +251,28 @@ namespace xt
 
         // without data_interface
         template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline auto get_flat_storage(E& e) -> flat_expression_adaptor<E>
+        inline auto get_flat_storage(E& e) -> flat_expression_adaptor<std::remove_reference_t<E>>
         {
-            return flat_expression_adaptor<E>(e);
+            // moved to addressof because ampersand on xview returns a closure pointer
+            return flat_expression_adaptor<std::remove_reference_t<E>>(std::addressof(e));
         }
 
         template <class E, class S>
-        inline auto get_flat_storage(E& e, S&& s, layout_type l) -> flat_expression_adaptor<E>
+        inline auto get_flat_storage(E& e, S&& s, layout_type l) -> flat_expression_adaptor<std::remove_reference_t<E>>
         {
-            return flat_expression_adaptor<E>(e, std::forward<S>(s), l);
+            return flat_expression_adaptor<std::remove_reference_t<E>>(std::addressof(e), std::forward<S>(s), l);
         }
 
         template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
-        inline std::size_t get_offset(E&& /*e*/)
+        inline auto get_offset(E&& /*e*/)
         {
-            return std::size_t(0);
+            return typename std::decay_t<E>::size_type(0);
         }
 
         template <class E, std::enable_if_t<!has_data_interface<std::decay_t<E>>::value>* = nullptr>
         inline auto get_strides(E&& e)
         {
-            dynamic_shape<std::size_t> strides;
+            dynamic_shape<std::ptrdiff_t> strides;
             strides.resize(e.shape().size());
             compute_strides(e.shape(), XTENSOR_DEFAULT_LAYOUT, strides);
             return strides;
@@ -269,7 +298,7 @@ namespace xt
      */
     template <class CT, class S, layout_type L, class FST>
     template <class CTA>
-    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(CTA&& e, S&& shape, S&& strides, std::size_t offset, layout_type layout) noexcept
+    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(CTA&& e, S&& shape, strides_type&& strides, size_type offset, layout_type layout) noexcept
         : m_e(std::forward<CTA>(e)),
           m_storage(detail::get_flat_storage<CT>(m_e)),
           m_shape(std::move(shape)),
@@ -283,8 +312,8 @@ namespace xt
 
     template <class CT, class S, layout_type L, class FST>
     template <class CTA, class FLS>
-    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(CTA&& e, S&& shape, S&& strides,
-                                                                 std::size_t offset, layout_type layout,
+    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(CTA&& e, S&& shape, strides_type&& strides,
+                                                                 size_type offset, layout_type layout,
                                                                  FLS&& flatten_strides, layout_type flatten_layout) noexcept
         : m_e(std::forward<CTA>(e)),
           m_storage(detail::get_flat_storage<CT>(m_e, std::forward<FLS>(flatten_strides), flatten_layout)),
@@ -296,6 +325,47 @@ namespace xt
         m_backstrides = xtl::make_sequence<backstrides_type>(m_shape.size(), 0);
         adapt_strides(m_shape, m_strides, m_backstrides);
     }
+
+    namespace detail
+    {
+        template <class T, class S>
+        auto& copy_move_storage(T& expr, S& /*storage*/)
+        {
+            return expr.storage();
+        }
+
+        template <class T, class E>
+        auto copy_move_storage(T& expr, const detail::flat_expression_adaptor<E>& storage)
+        {
+            detail::flat_expression_adaptor<E> new_storage = storage; // copy storage
+            new_storage.update_pointer(std::addressof(expr));
+            return new_storage;
+        }
+    }
+
+    template <class CT, class S, layout_type L, class FST>
+    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(xstrided_view_base&& rhs)
+        : m_e(std::forward<CT>(rhs.m_e)),
+          m_storage(detail::copy_move_storage(m_e, rhs.m_storage)),
+          m_shape(std::move(rhs.m_shape)),
+          m_strides(std::move(rhs.m_strides)),
+          m_backstrides(std::move(rhs.m_backstrides)),
+          m_offset(std::move(rhs.m_offset)),
+          m_layout(std::move(rhs.m_layout))
+    {
+    }
+
+    template <class CT, class S, layout_type L, class FST>
+    inline xstrided_view_base<CT, S, L, FST>::xstrided_view_base(const xstrided_view_base& rhs)
+        : m_e(rhs.m_e),
+          m_storage(detail::copy_move_storage(m_e, rhs.m_storage)),
+          m_shape(rhs.m_shape),
+          m_strides(rhs.m_strides),
+          m_backstrides(rhs.m_backstrides),
+          m_offset(rhs.m_offset),
+          m_layout(rhs.m_layout)
+    {
+    }
     //@}
 
     /**
@@ -324,7 +394,7 @@ namespace xt
      * Returns the shape of the xtrided_view_base.
      */
     template <class CT, class S, layout_type L, class FST>
-    inline auto xstrided_view_base<CT, S, L, FST>::shape() const noexcept -> const shape_type&
+    inline auto xstrided_view_base<CT, S, L, FST>::shape() const noexcept -> const inner_shape_type&
     {
         return m_shape;
     }
@@ -333,7 +403,7 @@ namespace xt
      * Returns the strides of the xtrided_view_base.
      */
     template <class CT, class S, layout_type L, class FST>
-    inline auto xstrided_view_base<CT, S, L, FST>::strides() const noexcept -> const strides_type&
+    inline auto xstrided_view_base<CT, S, L, FST>::strides() const noexcept -> const inner_strides_type&
     {
         return m_strides;
     }
@@ -342,7 +412,7 @@ namespace xt
      * Returns the backstrides of the xtrided_view_base.
      */
     template <class CT, class S, layout_type L, class FST>
-    inline auto xstrided_view_base<CT, S, L, FST>::backstrides() const noexcept -> const backstrides_type&
+    inline auto xstrided_view_base<CT, S, L, FST>::backstrides() const noexcept -> const inner_backstrides_type&
     {
         return m_backstrides;
     }
@@ -365,13 +435,13 @@ namespace xt
     template <class CT, class S, layout_type L, class FST>
     inline auto xstrided_view_base<CT, S, L, FST>::operator()() -> reference
     {
-        return m_storage[m_offset];
+        return m_storage[static_cast<size_type>(m_offset)];
     }
 
     template <class CT, class S, layout_type L, class FST>
     inline auto xstrided_view_base<CT, S, L, FST>::operator()() const -> const_reference
     {
-        return m_storage[m_offset];
+        return m_storage[static_cast<size_type>(m_offset)];
     }
 
     /**
@@ -386,8 +456,7 @@ namespace xt
     {
         XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
-        using offset_type = typename strides_type::value_type;
-        offset_type index = static_cast<offset_type>(m_offset) + xt::data_offset<offset_type>(strides(), static_cast<offset_type>(args)...);
+        offset_type index = compute_index(args...);
         return m_storage[static_cast<size_type>(index)];
     }
 
@@ -403,8 +472,7 @@ namespace xt
     {
         XTENSOR_TRY(check_index(shape(), args...));
         XTENSOR_CHECK_DIMENSION(shape(), args...);
-        using offset_type = typename strides_type::value_type;
-        offset_type index = static_cast<offset_type>(m_offset) + xt::data_offset<offset_type>(strides(), static_cast<offset_type>(args)...);
+        offset_type index = compute_index(args...);
         return m_storage[static_cast<size_type>(index)];
     }
 
@@ -465,8 +533,7 @@ namespace xt
     template <class... Args>
     inline auto xstrided_view_base<CT, S, L, FST>::unchecked(Args... args) -> reference
     {
-        using offset_type = typename strides_type::value_type;
-        offset_type index = static_cast<offset_type>(m_offset) + xt::unchecked_data_offset<offset_type>(strides(), static_cast<offset_type>(args)...);
+        offset_type index = compute_unchecked_index(args...);
         return m_storage[static_cast<size_type>(index)];
     }
 
@@ -493,8 +560,8 @@ namespace xt
     template <class... Args>
     inline auto xstrided_view_base<CT, S, L, FST>::unchecked(Args... args) const -> const_reference
     {
-        size_type index = m_offset + xt::unchecked_data_offset<size_type>(strides(), static_cast<size_type>(args)...);
-        return m_storage[index];
+        offset_type index = compute_unchecked_index(args...);
+        return m_storage[static_cast<size_type>(index)];
     }
 
     /**
@@ -565,7 +632,7 @@ namespace xt
     inline auto xstrided_view_base<CT, S, L, FST>::element(It first, It last) -> reference
     {
         XTENSOR_TRY(check_element_index(shape(), first, last));
-        return m_storage[m_offset + element_offset<size_type>(strides(), first, last)];
+        return m_storage[static_cast<size_type>(compute_element_index(first, last))];
     }
 
     /**
@@ -580,7 +647,7 @@ namespace xt
     inline auto xstrided_view_base<CT, S, L, FST>::element(It first, It last) const -> const_reference
     {
         XTENSOR_TRY(check_element_index(shape(), first, last));
-        return m_storage[m_offset + element_offset<size_type>(strides(), first, last)];
+        return m_storage[static_cast<size_type>(compute_element_index(first, last))];
     }
 
     /**
@@ -679,37 +746,58 @@ namespace xt
     template <class O>
     inline bool xstrided_view_base<CT, S, L, FST>::is_trivial_broadcast(const O& str) const noexcept
     {
-        return str.size() == strides().size() &&
+        return has_data_interface<std::decay_t<CT>>::value && str.size() == strides().size() &&
             std::equal(str.cbegin(), str.cend(), strides().begin());
     }
     //@}
 
+    template <class CT, class S, layout_type L, class FST>
+    template <class... Args>
+    inline auto xstrided_view_base<CT, S, L, FST>::compute_index(Args... args) const -> offset_type
+    {
+        return static_cast<offset_type>(m_offset) + xt::data_offset<offset_type>(strides(), static_cast<offset_type>(args)...);
+    }
+
+    template <class CT, class S, layout_type L, class FST>
+    template <class... Args>
+    inline auto xstrided_view_base<CT, S, L, FST>::compute_unchecked_index(Args... args) const -> offset_type
+    {
+        return static_cast<offset_type>(m_offset) + xt::unchecked_data_offset<offset_type>(strides(), static_cast<offset_type>(args)...);
+    }
+
+    template <class CT, class S, layout_type L, class FST>
+    template <class It>
+    inline auto xstrided_view_base<CT, S, L, FST>::compute_element_index(It first, It last) const -> offset_type
+    {
+        return static_cast<offset_type>(m_offset) + xt::element_offset<offset_type>(strides(), first, last);
+    }
+
     /******************************************
      * flat_expression_adaptor implementation *
      ******************************************/
 
-        namespace detail
+    namespace detail
     {
         template <class CT>
-        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT& e)
+        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT* e)
             : m_e(e)
         {
-            resize_container(m_index, m_e.dimension());
-            resize_container(m_strides, m_e.dimension());
-            m_size = compute_size(m_e.shape());
+            resize_container(m_index, m_e->dimension());
+            resize_container(m_strides, m_e->dimension());
+            m_size = compute_size(m_e->shape());
             // Fallback to XTENSOR_DEFAULT_LAYOUT when the underlying layout is not
             // row-major or column major.
-            m_layout = default_assignable_layout(m_e.layout());
-            compute_strides(m_e.shape(), m_layout, m_strides);
+            m_layout = default_assignable_layout(m_e->layout());
+            compute_strides(m_e->shape(), m_layout, m_strides);
         }
 
         template <class CT>
         template <class FST>
-        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT& e, FST&& strides, layout_type layout)
+        inline flat_expression_adaptor<CT>::flat_expression_adaptor(CT* e, FST&& strides, layout_type layout)
             : m_e(e), m_strides(xtl::forward_sequence<shape_type>(strides)), m_layout(layout)
         {
-            resize_container(m_index, m_e.dimension());
-            m_size = e.size();
+            resize_container(m_index, m_e->dimension());
+            m_size = m_e->size();
         }
 
         template <class CT>
@@ -719,55 +807,223 @@ namespace xt
         }
 
         template <class CT>
-        inline auto flat_expression_adaptor<CT>::operator[](std::size_t idx) -> reference
+        inline auto flat_expression_adaptor<CT>::operator[](size_type idx) -> reference
         {
             m_index = detail::unravel_noexcept(idx, m_strides, m_layout);
-            return m_e.element(m_index.cbegin(), m_index.cend());
+            return m_e->element(m_index.cbegin(), m_index.cend());
         }
 
         template <class CT>
-        inline auto flat_expression_adaptor<CT>::operator[](std::size_t idx) const -> const_reference
+        inline auto flat_expression_adaptor<CT>::operator[](size_type idx) const -> const_reference
         {
             m_index = detail::unravel_noexcept(idx, m_strides, m_layout);
-            return m_e.element(m_index.cbegin(), m_index.cend());
+            return m_e->element(m_index.cbegin(), m_index.cend());
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::begin() -> iterator
         {
-            return m_e.begin();
+            return m_e->begin();
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::end() -> iterator
         {
-            return m_e.end();
+            return m_e->end();
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::begin() const -> const_iterator
         {
-            return m_e.cbegin();
+            return m_e->cbegin();
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::end() const -> const_iterator
         {
-            return m_e.cend();
+            return m_e->cend();
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::cbegin() const -> const_iterator
         {
-            return m_e.cbegin();
+            return m_e->cbegin();
         }
 
         template <class CT>
         inline auto flat_expression_adaptor<CT>::cend() const ->const_iterator
         {
-            return m_e.cend();
+            return m_e->cend();
         }
     }
+
+    /**********************************
+     * Builder helpers implementation *
+     **********************************/
+
+    namespace detail
+    {
+        template <class S>
+        struct slice_getter_impl
+        {
+            const S& m_shape;
+            mutable std::size_t idx;
+
+            explicit slice_getter_impl(const S& shape)
+                : m_shape(shape), idx(0)
+            {
+            }
+
+            template <class T>
+            std::array<std::ptrdiff_t, 3> operator()(const T& /*t*/) const
+            {
+                return std::array<std::ptrdiff_t, 3>{{0, 0, 0}};
+            }
+
+            template <class A, class B, class C>
+            std::array<std::ptrdiff_t, 3> operator()(const xrange_adaptor<A, B, C>& range) const
+            {
+                auto sl = range.get(static_cast<std::size_t>(m_shape[idx]));
+                return std::array<std::ptrdiff_t, 3>({sl(0), sl.size(), sl.step_size()});
+            }
+        };
+
+        template <class adj_strides_policy>
+        struct strided_view_args : adj_strides_policy
+        {
+            using base_type = adj_strides_policy;
+
+            template <class S, class ST, class V>
+            void fill_args(const S& shape, ST&& old_strides, std::size_t base_offset, layout_type layout, const V& slices)
+            {
+                // Compute dimension
+                std::size_t dimension = shape.size(), n_newaxis = 0, n_add_all = 0;
+                std::ptrdiff_t dimension_check = static_cast<std::ptrdiff_t>(shape.size());
+
+                bool has_ellipsis = false;
+                for (const auto& el : slices)
+                {
+                    if (xtl::get_if<xt::xnewaxis_tag>(&el) != nullptr)
+                    {
+                        ++dimension;
+                        ++n_newaxis;
+                    }
+                    else if (xtl::get_if<std::ptrdiff_t>(&el) != nullptr)
+                    {
+                        --dimension;
+                        --dimension_check;
+                    }
+                    else if (xtl::get_if<xt::xellipsis_tag>(&el) != nullptr)
+                    {
+                        if (has_ellipsis == true)
+                        {
+                            throw std::runtime_error("Ellipsis can only appear once.");
+                        }
+                        has_ellipsis = true;
+                    }
+                    else
+                    {
+                        --dimension_check;
+                    }
+                }
+
+                if (dimension_check < 0)
+                {
+                    throw std::runtime_error("Too many slices for view.");
+                }
+
+                if (has_ellipsis)
+                {
+                    // replace ellipsis with N * xt::all
+                    // remove -1 because of the ellipsis slize itself
+                    n_add_all = shape.size() - (slices.size() - 1 - n_newaxis);
+                }
+
+                // Compute strided view
+                new_offset = base_offset;
+                new_shape.resize(dimension);
+                new_strides.resize(dimension);
+                base_type::resize(dimension);
+
+                auto old_shape = shape;
+                using old_strides_vt = std::decay_t<decltype(old_strides[0])>;
+
+                std::ptrdiff_t axis_skip = 0;
+                std::size_t idx = 0, i = 0, i_ax = 0;
+
+                auto slice_getter = detail::slice_getter_impl<S>(shape);
+
+                for (; i < slices.size(); ++i)
+                {
+                    i_ax = static_cast<std::size_t>(static_cast<std::ptrdiff_t>(i) - axis_skip);
+                    auto ptr = xtl::get_if<std::ptrdiff_t>(&slices[i]);
+                    if (ptr != nullptr)
+                    {
+                        auto slice0 = static_cast<old_strides_vt>(*ptr);
+                        new_offset += static_cast<std::size_t>(slice0 * old_strides[i_ax]);
+                    }
+                    else if (xtl::get_if<xt::xnewaxis_tag>(&slices[i]) != nullptr)
+                    {
+                        new_shape[idx] = 1;
+                        base_type::set_fake_slice(idx);
+                        ++axis_skip, ++idx;
+                    }
+                    else if (xtl::get_if<xt::xellipsis_tag>(&slices[i]) != nullptr)
+                    {
+                        for (std::size_t j = 0; j < n_add_all; ++j)
+                        {
+                            new_shape[idx] = old_shape[i_ax];
+                            new_strides[idx] = old_strides[i_ax];
+                            base_type::set_fake_slice(idx);
+                            ++idx, ++i_ax;
+                        }
+                        axis_skip = axis_skip - static_cast<std::ptrdiff_t>(n_add_all) + 1;
+                    }
+                    else if (xtl::get_if<xt::xall_tag>(&slices[i]) != nullptr)
+                    {
+                        new_shape[idx] = old_shape[i_ax];
+                        new_strides[idx] = old_strides[i_ax];
+                        base_type::set_fake_slice(idx);
+                        ++idx;
+                    }
+                    else if (base_type::fill_args(slices, i, idx,
+                                                  old_shape[i_ax],
+                                                  old_strides[i_ax],
+                                                  new_shape, new_strides))
+                    {
+                        ++idx;
+                    }
+                    else
+                    {
+                        slice_getter.idx = i_ax;
+                        auto info = xtl::visit(slice_getter, slices[i]);
+                        new_offset += static_cast<std::size_t>(info[0] * old_strides[i_ax]);
+                        new_shape[idx] = static_cast<std::size_t>(info[1]);
+                        new_strides[idx] = info[2] * old_strides[i_ax];
+                        base_type::set_fake_slice(idx);
+                        ++idx;
+                    }
+                }
+
+                i_ax = static_cast<std::size_t>(static_cast<std::ptrdiff_t>(i) - axis_skip);
+                for (; i_ax < old_shape.size(); ++i_ax, ++idx)
+                {
+                    new_shape[idx] = old_shape[i_ax];
+                    new_strides[idx] = old_strides[i_ax];
+                    base_type::set_fake_slice(idx);
+                }
+
+                new_layout = do_strides_match(new_shape, new_strides, layout) ? layout : layout_type::dynamic;
+            }
+
+            using shape_type = dynamic_shape<std::size_t>;
+            shape_type new_shape;
+            using strides_type = dynamic_shape<std::ptrdiff_t>;
+            strides_type new_strides;
+            std::size_t new_offset;
+            layout_type new_layout;
+        };
+    }
 }
 
 #endif
