@@ -26,26 +26,26 @@ namespace xt
      * data offset *
      ***************/
 
-    template <class size_type, class S>
-    size_type data_offset(const S& strides) noexcept;
+    template <class offset_type, class S>
+    offset_type data_offset(const S& strides) noexcept;
 
-    template <class size_type, class S, class Arg, class... Args>
-    size_type data_offset(const S& strides, Arg arg, Args... args) noexcept;
+    template <class offset_type, class S, class Arg, class... Args>
+    offset_type data_offset(const S& strides, Arg arg, Args... args) noexcept;
 
-    template <class size_type, class S, class... Args>
-    size_type unchecked_data_offset(const S& strides, Args... args) noexcept;
+    template <class offset_type, layout_type L = layout_type::dynamic, class S, class... Args>
+    offset_type unchecked_data_offset(const S& strides, Args... args) noexcept;
 
-    template <class size_type, class S, class It>
-    size_type element_offset(const S& strides, It first, It last) noexcept;
+    template <class offset_type, class S, class It>
+    offset_type element_offset(const S& strides, It first, It last) noexcept;
 
     /*******************
      * strides builder *
      *******************/
 
-    template <class shape_type, class strides_type>
+    template <layout_type L = layout_type::dynamic, class shape_type, class strides_type>
     std::size_t compute_strides(const shape_type& shape, layout_type l, strides_type& strides);
 
-    template <class shape_type, class strides_type, class backstrides_type>
+    template <layout_type L = layout_type::dynamic, class shape_type, class strides_type, class backstrides_type>
     std::size_t compute_strides(const shape_type& shape, layout_type l,
                                 strides_type& strides, backstrides_type& backstrides);
 
@@ -64,7 +64,7 @@ namespace xt
     S unravel_from_strides(typename S::value_type index, const S& strides, layout_type l);
 
     template <class S>
-    S unravel_index(typename S::value_type index, const S& shape, layout_type l);
+    get_strides_t<S> unravel_index(typename S::value_type index, const S& shape, layout_type l);
 
     /***********************
      * broadcast functions *
@@ -115,60 +115,127 @@ namespace xt
 
     namespace detail
     {
-        template <class size_type, std::size_t dim, class S>
-        inline size_type raw_data_offset(const S&) noexcept
+        template <std::size_t dim, class S>
+        inline auto raw_data_offset(const S&) noexcept
         {
-            return 0;
+            using strides_value_type = std::decay_t<decltype(std::declval<S>()[0])>;
+            return strides_value_type(0);
         }
 
-        template <class size_type, std::size_t dim, class S, class Arg, class... Args>
-        inline size_type raw_data_offset(const S& strides, Arg arg, Args... args) noexcept
+        template <std::size_t dim, class S, class Arg, class... Args>
+        inline auto raw_data_offset(const S& strides, Arg arg, Args... args) noexcept
         {
-            using strides_type = decltype(strides[0]);
-            return strides_type(arg) * strides[dim] + raw_data_offset<size_type, dim + 1>(strides, args...);
+            return arg * strides[dim] + raw_data_offset<dim + 1>(strides, args...);
         }
+
+        template <layout_type L, std::ptrdiff_t static_dim>
+        struct layout_data_offset
+        {
+            template <std::size_t dim, class S, class Arg, class... Args>
+            static inline auto run(const S& strides, Arg arg, Args... args) noexcept
+            {
+                return raw_data_offset<dim>(strides, arg, args...);
+            }
+        };
+
+        template <std::ptrdiff_t static_dim>
+        struct layout_data_offset<layout_type::row_major, static_dim>
+        {
+            using self_type = layout_data_offset<layout_type::row_major, static_dim>;
+
+            template <std::size_t dim, class S, class Arg>
+            static inline auto run(const S& strides, Arg arg) noexcept
+            {
+                if (std::ptrdiff_t(dim) + 1 == static_dim)
+                {
+                    return arg;
+                }
+                else
+                {
+                    return arg * strides[dim];
+                }
+            }
+
+            template <std::size_t dim, class S, class Arg, class... Args>
+            static inline auto run(const S& strides, Arg arg, Args... args) noexcept
+            {
+                return arg * strides[dim] + self_type::template run<dim + 1>(strides, args...);
+            }
+        };
+
+        template <std::ptrdiff_t static_dim>
+        struct layout_data_offset<layout_type::column_major, static_dim>
+        {
+            using self_type = layout_data_offset<layout_type::column_major, static_dim>;
+
+            template <std::size_t dim, class S, class Arg>
+            static inline auto run(const S& strides, Arg arg) noexcept
+            {
+                if (dim == 0)
+                {
+                    return arg;
+                }
+                else
+                {
+                    return arg * strides[dim];
+                }
+            }
+
+            template <std::size_t dim, class S, class Arg, class... Args>
+            static inline auto run(const S& strides, Arg arg, Args... args) noexcept
+            {
+                if (dim == 0)
+                {
+                    return arg + self_type::template run<dim + 1>(strides, args...);
+                }
+                else
+                {
+                    return arg * strides[dim] + self_type::template run<dim + 1>(strides, args...);
+                }
+            }
+        };
     }
 
-    template <class size_type, class S>
-    inline size_type data_offset(const S&) noexcept
+    template <class offset_type, class S>
+    inline offset_type data_offset(const S&) noexcept
     {
-        return 0;
+        return offset_type(0);
     }
 
-    template <class size_type, class S, class Arg, class... Args>
-    inline size_type data_offset(const S& strides, Arg arg, Args... args) noexcept
+    template <class offset_type, class S, class Arg, class... Args>
+    inline offset_type data_offset(const S& strides, Arg arg, Args... args) noexcept
     {
         constexpr std::size_t nargs = sizeof...(Args) + 1;
         if (nargs == strides.size())
         {
             // Correct number of arguments: iterate
-            return detail::raw_data_offset<size_type, 0>(strides, arg, args...);
+            return static_cast<offset_type>(detail::raw_data_offset<0>(strides, arg, args...));
         }
         else if (nargs > strides.size())
         {
             // Too many arguments: drop the first
-            return data_offset<size_type, S>(strides, args...);
+            return data_offset<offset_type, S>(strides, args...);
         }
         else
         {
             // Too few arguments: right to left scalar product
             auto view = strides.cend() - nargs;
-            return detail::raw_data_offset<size_type, 0>(view, arg, args...);
+            return static_cast<offset_type>(detail::raw_data_offset<0>(view, arg, args...));
         }
     }
 
-    template <class size_type, class S, class... Args>
-    inline size_type unchecked_data_offset(const S& strides, Args... args) noexcept
+    template <class offset_type, layout_type L, class S, class... Args>
+    inline offset_type unchecked_data_offset(const S& strides, Args... args) noexcept
     {
-        return detail::raw_data_offset<size_type, 0>(strides.cbegin(), args...);
+        return static_cast<offset_type>(detail::layout_data_offset<L, static_dimension<S>::value>::template run<0>(strides.cbegin(), args...));
     }
 
-    template <class size_type, class S, class It>
-    inline size_type element_offset(const S& strides, It first, It last) noexcept
+    template <class offset_type, class S, class It>
+    inline offset_type element_offset(const S& strides, It first, It last) noexcept
     {
         using difference_type = typename std::iterator_traits<It>::difference_type;
         auto size = static_cast<difference_type>((std::min)(static_cast<typename S::size_type>(std::distance(first, last)), strides.size()));
-        return std::inner_product(last - size, last, strides.cend() - size, size_type(0));
+        return std::inner_product(last - size, last, strides.cend() - size, offset_type(0));
     }
 
     namespace detail
@@ -181,7 +248,7 @@ namespace xt
             {
                 strides[i] = 0;
             }
-            (*backstrides)[i] = strides[i] * (shape[i] - 1);
+            (*backstrides)[i] = strides[i] * std::ptrdiff_t(shape[i] - 1);
         }
 
         template <class shape_type, class strides_type>
@@ -194,52 +261,53 @@ namespace xt
             }
         }
 
-        template <class shape_type, class strides_type, class bs_ptr>
+        template <layout_type L, class shape_type, class strides_type, class bs_ptr>
         inline std::size_t compute_strides(const shape_type& shape, layout_type l,
                                            strides_type& strides, bs_ptr bs)
         {
-            std::size_t data_size = 1;
-            if (l == layout_type::row_major)
+            using strides_value_type = std::decay_t<decltype(strides[0])>;
+            strides_value_type data_size = 1;
+            if (L == layout_type::row_major || l == layout_type::row_major)
             {
-                for (std::size_t i = strides.size(); i != 0; --i)
+                for (std::size_t i = shape.size(); i != 0; --i)
                 {
                     strides[i - 1] = data_size;
-                    data_size = strides[i - 1] * shape[i - 1];
+                    data_size = strides[i - 1] * static_cast<strides_value_type>(shape[i - 1]);
                     adapt_strides(shape, strides, bs, i - 1);
                 }
             }
             else
             {
-                for (std::size_t i = 0; i < strides.size(); ++i)
+                for (std::size_t i = 0; i < shape.size(); ++i)
                 {
                     strides[i] = data_size;
-                    data_size = strides[i] * shape[i];
+                    data_size = strides[i] * static_cast<strides_value_type>(shape[i]);
                     adapt_strides(shape, strides, bs, i);
                 }
             }
-            return data_size;
+            return static_cast<std::size_t>(data_size);
         }
     }
 
-    template <class shape_type, class strides_type>
+    template <layout_type L, class shape_type, class strides_type>
     inline std::size_t compute_strides(const shape_type& shape, layout_type l, strides_type& strides)
     {
-        return detail::compute_strides(shape, l, strides, nullptr);
+        return detail::compute_strides<L>(shape, l, strides, nullptr);
     }
 
-    template <class shape_type, class strides_type, class backstrides_type>
+    template <layout_type L, class shape_type, class strides_type, class backstrides_type>
     inline std::size_t compute_strides(const shape_type& shape, layout_type l,
                                        strides_type& strides,
                                        backstrides_type& backstrides)
     {
-        return detail::compute_strides(shape, l, strides, &backstrides);
+        return detail::compute_strides<L>(shape, l, strides, &backstrides);
     }
 
     template <class shape_type, class strides_type>
     inline bool do_strides_match(const shape_type& shape, const strides_type& strides, layout_type l)
     {
-        using shape_value_type = typename shape_type::value_type;
-        shape_value_type data_size = 1;
+        using value_type = typename strides_type::value_type;
+        value_type data_size = 1;
         if (l == layout_type::row_major)
         {
             for (std::size_t i = strides.size(); i != 0; --i)
@@ -248,7 +316,7 @@ namespace xt
                 {
                     return false;
                 }
-                data_size *= shape[i - 1];
+                data_size *= static_cast<value_type>(shape[i - 1]);
             }
             return true;
         }
@@ -260,7 +328,7 @@ namespace xt
                 {
                     return false;
                 }
-                data_size *= shape[i];
+                data_size *= static_cast<value_type>(shape[i]);
             }
             return true;
         }
@@ -332,11 +400,12 @@ namespace xt
     }
 
     template <class S>
-    inline S unravel_index(typename S::value_type index, const S& shape, layout_type l)
+    inline get_strides_t<S> unravel_index(typename S::value_type index, const S& shape, layout_type l)
     {
-        S strides = xtl::make_sequence<S>(shape.size(), 0);
+        get_strides_t<S> strides = xtl::make_sequence<get_strides_t<S>>(shape.size(), 0);
+        using strides_value_type = std::decay_t<decltype(strides[0])>;
         compute_strides(shape, l, strides);
-        return unravel_from_strides(index, strides, l);
+        return unravel_from_strides(static_cast<strides_value_type>(index), strides, l);
     }
 
     template <class S1, class S2>
@@ -347,6 +416,7 @@ namespace xt
         using value_type = typename S2::value_type;
         auto output_index = output.size();
         auto input_index = input.size();
+
         if (output_index < input_index)
         {
             throw_broadcast_error(output, input);
